@@ -105,9 +105,19 @@ class DataSource(metaclass=ABCMeta):
 
     @property
     def threads(self):
+        try:
+            engine = self.get_engine_by_database()
+        except Exception:
+            engine = None
         if self.credential.get('threads'):
-            return self.credential.get('threads')
-        elif not isinstance(self.get_engine_by_database().pool, SingletonThreadPool):
+            try:
+                return int(self.credential.get('threads'))
+            except BaseException:
+                console = Console()
+                message = 'failed to parse the "threads" field, so we will use 1 thread to execute profiling.'
+                console.print(f'[bold yellow]Warning: [/bold yellow]:\n  {message}')
+                return 1
+        elif engine and not isinstance(engine.pool, SingletonThreadPool):
             return 5
         else:
             return 1
@@ -267,11 +277,15 @@ def _list_datasource_providers():
     from .survey import UserSurveyMockDataSource
     from .duckdb import DuckDBDataSource, CsvDataSource, ParquetDataSource
     from .athena import AthenaDataSource
+    from .databricks import DataBricksDataSource
+    from .unsupported import UnsupportedDataSource
+
     return {
         'snowflake': SnowflakeDataSource,
         'bigquery': BigQueryDataSource,
         'redshift': RedshiftDataSource,
         'athena': AthenaDataSource,
+        'databricks': DataBricksDataSource,
         'postgres': PostgresDataSource,
         'sqlite': SqliteDataSource,
         'duckdb': DuckDBDataSource,
@@ -281,4 +295,10 @@ def _list_datasource_providers():
     }
 
 
-DATASOURCE_PROVIDERS = _list_datasource_providers()
+class DataSourceProviders(dict):
+    def __getitem__(self, key):
+        from .unsupported import UnsupportedDataSource
+        return super().get(key, UnsupportedDataSource)
+
+
+DATASOURCE_PROVIDERS = DataSourceProviders(_list_datasource_providers())
