@@ -3,6 +3,7 @@ import os
 import re
 import shutil
 from base64 import b64encode
+from typing import Dict
 
 from rich.console import Console
 
@@ -10,11 +11,10 @@ from piperider_cli import __version__, open_report_in_browser, sentry_dns, sentr
 from piperider_cli import clone_directory, raise_exception_when_directory_not_writable
 from piperider_cli.configuration import Configuration
 from piperider_cli.error import PipeRiderNoProfilingResultError
-from piperider_cli.filesystem import FileSystem
 
 
 def prepare_piperider_metadata():
-    configuration = Configuration.load()
+    configuration = Configuration.instance()
     project_id = configuration.get_telemetry_id()
     metadata = {
         'name': 'PipeRider',
@@ -60,7 +60,7 @@ def setup_report_variables(template_html: str, is_single: bool, data):
 
 def _generate_static_html(result, html, output_path):
     filename = os.path.join(output_path, "index.html")
-    with open(filename, 'w') as f:
+    with open(filename, 'w', encoding='utf-8') as f:
         html = setup_report_variables(html, True, result)
         f.write(html)
 
@@ -68,7 +68,7 @@ def _generate_static_html(result, html, output_path):
 class GenerateReport:
     @staticmethod
     def exec(input=None, report_dir=None, output=None, open_report=None, open_in_cloud=None):
-        filesystem = FileSystem(report_dir=report_dir)
+        filesystem = Configuration.instance().activate_report_directory(report_dir)
         raise_exception_when_directory_not_writable(output)
 
         console = Console()
@@ -79,6 +79,8 @@ class GenerateReport:
             report_template_html = f.read()
 
         run_json_path = get_run_json_path(filesystem.get_output_dir(), input)
+        if os.path.islink(os.path.dirname(run_json_path)):
+            run_json_path = os.path.realpath(run_json_path)
         if not os.path.isfile(run_json_path):
             print(os.path.abspath(run_json_path))
             raise PipeRiderNoProfilingResultError(run_json_path)
@@ -89,7 +91,8 @@ class GenerateReport:
             console.print(f'[bold red]Error: {run_json_path} is invalid[/bold red]')
             return
 
-        console.print(f'[bold dark_orange]Generating reports from:[/bold dark_orange] {run_json_path}')
+        console.print('')
+        console.print(f'Generating reports from: {run_json_path}', soft_wrap=True)
 
         def output_report(target_directory):
             clone_directory(report_template_dir, target_directory)
@@ -102,9 +105,11 @@ class GenerateReport:
         if output:
             output_report(output)
             shutil.copyfile(run_json_path, os.path.join(output, os.path.basename(run_json_path)))
-            console.print(f"Report generated in {os.path.join(output, 'index.html')}")
+            console.print(
+                f"Report generated in: {os.path.join(output, 'index.html')}", soft_wrap=True)
         else:
-            console.print(f"Report generated in {os.path.join(default_output_directory, 'index.html')}")
+            console.print(
+                f"Report generated in: {os.path.join(default_output_directory, 'index.html')}", soft_wrap=True)
 
         # only open the local file report if auto-upload is OFF
         if open_report and not open_in_cloud:
