@@ -7,15 +7,15 @@ from rich.prompt import Prompt
 from rich.syntax import Syntax
 from rich.table import Table
 
-from piperider_cli import clone_directory, safe_load_yaml
-from piperider_cli.configuration import Configuration, \
-    PIPERIDER_WORKSPACE_NAME, \
-    PIPERIDER_CONFIG_PATH, \
-    PIPERIDER_CREDENTIALS_PATH
+from piperider_cli import clone_directory
+from piperider_cli.yaml import safe_load_yaml
+
+from piperider_cli.configuration import Configuration, FileSystem
 from piperider_cli.datasource import DataSource, FANCY_USER_INPUT
 from piperider_cli.datasource.survey import UserSurveyMockDataSource
 from piperider_cli.error import PipeRiderConfigError
-from piperider_cli.recipes.default_recipe_generator import generate_default_recipe, show_recipe_content
+
+console = Console()
 
 
 def _is_piperider_workspace_exist(workspace_path: str) -> bool:
@@ -30,7 +30,7 @@ def _is_piperider_workspace_exist(workspace_path: str) -> bool:
 def _generate_piperider_workspace() -> bool:
     from piperider_cli import data
     init_template_dir = os.path.join(os.path.dirname(data.__file__), 'piperider-init-template')
-    working_dir = os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME)
+    working_dir = FileSystem.PIPERIDER_WORKSPACE_PATH
 
     if not _is_piperider_workspace_exist(working_dir):
         clone_directory(init_template_dir, working_dir)
@@ -43,21 +43,19 @@ def _generate_piperider_workspace() -> bool:
 
 
 def _ask_user_update_credentials(ds: DataSource):
-    console = Console()
     console.print(f'\nPlease enter the following fields for {ds.type_name}')
     return ds.ask_credential()
 
 
 def _ask_user_input_datasource(config: Configuration = None):
-    console = Console()
     if config is None:
         cls, name = DataSource.ask()
         ds: DataSource = cls(name=name)
         config = Configuration([ds])
         if _ask_user_update_credentials(ds):
             _generate_piperider_workspace()
-            config.dump(PIPERIDER_CONFIG_PATH)
-            config.dump_credentials(PIPERIDER_CREDENTIALS_PATH, after_init_config=True)
+            config.dump(FileSystem.PIPERIDER_CONFIG_PATH)
+            config.dump_credentials(FileSystem.PIPERIDER_CREDENTIALS_PATH, after_init_config=True)
     else:
         if len(config.dataSources) == 1:
             ds = config.dataSources[0]
@@ -67,7 +65,7 @@ def _ask_user_input_datasource(config: Configuration = None):
             console.print(
                 f'[[bold yellow]Warning[/bold yellow]] No credential found for \'{ds.type_name}\' datasource \'{ds.name}\'')
             if _ask_user_update_credentials(ds):
-                config.dump_credentials(PIPERIDER_CREDENTIALS_PATH)
+                config.dump_credentials(FileSystem.PIPERIDER_CREDENTIALS_PATH)
 
     ds.show_installation_information()
     if isinstance(ds, UserSurveyMockDataSource):
@@ -76,77 +74,90 @@ def _ask_user_input_datasource(config: Configuration = None):
     return config
 
 
-def _inherit_datasource_from_dbt_project(dbt_project_path, dbt_profiles_dir=None):
-    config = safe_load_yaml(PIPERIDER_CONFIG_PATH)
+def _inherit_datasource_from_dbt_project(dbt_project_path, dbt_profiles_dir=None, dbt_profile: str = None,
+                                         dbt_target: str = None, interactive=True):
+    config = safe_load_yaml(FileSystem.PIPERIDER_CONFIG_PATH)
     if config and config.get('dataSources'):
-        console = Console()
-        console.print('[[bold yellow]Warning[/bold yellow]] Found existing configuration. Skip initialization.')
+        if interactive is True:
+            console.print('[[bold yellow]Warning[/bold yellow]] Found existing configuration. Skip initialization.')
         return config
 
-    dbt_config = Configuration.from_dbt_project(dbt_project_path, dbt_profiles_dir)
+    dbt_config = Configuration.from_dbt_project(dbt_project_path, dbt_profiles_dir, dbt_profile=dbt_profile,
+                                                dbt_target=dbt_target)
     _generate_piperider_workspace()
-    dbt_config.dump(PIPERIDER_CONFIG_PATH)
+    dbt_config.dump(FileSystem.PIPERIDER_CONFIG_PATH)
+
+    console.print(f'[[bold green] DBT [/bold green]] Use the existing dbt project file: {dbt_project_path}')
 
     return dbt_config
 
 
-def _generate_configuration(dbt_project_path=None, dbt_profiles_dir=None):
+def _generate_configuration(dbt_project_path=None, dbt_profiles_dir=None, dbt_profile: str = None,
+                            dbt_target: str = None, interactive=True, ):
     """
     :param dbt_project_path:
     :return: Configuration object
     """
     try:
-        config = Configuration.load()
+        config = Configuration.instance()
     except PipeRiderConfigError:
         config = None
     except Exception:
         config = None
-        console = Console()
         console.print('[[bold yellow]Warning[/bold yellow]] Invalid config.yml')
+
     if dbt_project_path is None:
+        # TODO: mark as deprecated in the future
+        console.rule('Deprecated', style='bold red')
+        console.print(
+            'Non-dbt project is deprecated and will be removed in the future. If you have a strong need for non-dbt '
+            'project, please contact us by "piperider feedback".\n')
         return _ask_user_input_datasource(config=config)
 
-    return _inherit_datasource_from_dbt_project(dbt_project_path, dbt_profiles_dir)
+    if config is not None:
+        if interactive:
+            console.print('[[bold yellow]Warning[/bold yellow]] Found existing configuration. Skip initialization.')
+        return config
+
+    return _inherit_datasource_from_dbt_project(dbt_project_path, dbt_profiles_dir, interactive=interactive,
+                                                dbt_profile=dbt_profile,
+                                                dbt_target=dbt_target)
 
 
-class Initializer():
+class Initializer:
     @staticmethod
-    def exec(working_dir=None, dbt_project_path=None, dbt_profiles_dir=None):
-        console = Console()
+    def exec(working_dir=None, dbt_project_path=None, dbt_profiles_dir=None, dbt_profile: str = None,
+             dbt_target: str = None, interactive=True, reload: bool = True):
         if working_dir is None:
-            working_dir = os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME)
+            working_dir = FileSystem.PIPERIDER_WORKSPACE_PATH
 
-        if _is_piperider_workspace_exist(working_dir):
+        if _is_piperider_workspace_exist(working_dir) and interactive is True:
             console.print('[bold green]Piperider workspace already exist[/bold green] ')
 
         # get Configuration object from dbt or user created configuration
-        configuration = _generate_configuration(dbt_project_path, dbt_profiles_dir)
+        _generate_configuration(dbt_project_path, dbt_profiles_dir, interactive=interactive, dbt_profile=dbt_profile,
+                                dbt_target=dbt_target)
+        configuration = Configuration.instance(dbt_profile=dbt_profile, dbt_target=dbt_target, reload=reload)
+        configuration.activate_report_directory()
 
-        # generate the default recipe
-        generate_default_recipe(dbt_project_path=dbt_project_path)
         return configuration
 
     @staticmethod
-    def show_config():
-        console = Console()
+    def show_config_file():
 
         # show config.yml
-        with open(PIPERIDER_CONFIG_PATH, 'r') as f:
+        with open(FileSystem.PIPERIDER_CONFIG_PATH, 'r') as f:
             console.rule('.piperider/config.yml')
             config = Syntax(f.read(), "yaml", theme="monokai", line_numbers=True)
             console.print(config)
             console.rule('End of .piperider/config.yml')
 
-        # show default recipe
-        show_recipe_content()
-
     @staticmethod
     def list(report_dir=None):
-        console = Console()
-        working_dir = os.path.join(os.getcwd(), PIPERIDER_WORKSPACE_NAME)
+        working_dir = FileSystem.PIPERIDER_WORKSPACE_PATH
 
         if _is_piperider_workspace_exist(working_dir):
-            config = Configuration.load()
+            config = Configuration.instance()
             list_table = Table(show_header=True, show_edge=True, box=box.SIMPLE_HEAVY)
             list_table.add_column("Datasource", style="cyan", no_wrap=True)
             list_table.add_column("Description", style="magenta", no_wrap=True)
@@ -166,9 +177,8 @@ class Initializer():
 
     @staticmethod
     def delete(report_dir=None):
-        console = Console()
 
-        config = Configuration.load()
+        config = Configuration.instance()
         if config.dbt:
             console.print('[bold yellow]You have connected with a dbt project. '
                           'Please add datasource in the dbt profile directly. [/bold yellow]')
@@ -199,13 +209,12 @@ class Initializer():
             return
 
         config.delete_datasource(answers['datasource'])
-        config.flush_datasource(PIPERIDER_CONFIG_PATH)
+        config.flush_datasource(FileSystem.PIPERIDER_CONFIG_PATH)
         console.rule('Datasource deleted')
 
     @staticmethod
     def add(report_dir=None):
-        console = Console()
-        config = Configuration.load()
+        config = Configuration.instance()
         if config.dbt:
             console.print('[bold yellow]You have connected with a dbt project. '
                           'Please add datasource in the dbt profile directly. [/bold yellow]')
@@ -216,8 +225,8 @@ class Initializer():
         config.dataSources.append(ds)
         if _ask_user_update_credentials(ds):
             _generate_piperider_workspace()
-            config.flush_datasource(PIPERIDER_CONFIG_PATH)
-            config.dump_credentials(PIPERIDER_CREDENTIALS_PATH, after_init_config=True)
+            config.flush_datasource(FileSystem.PIPERIDER_CONFIG_PATH)
+            config.dump_credentials(FileSystem.PIPERIDER_CREDENTIALS_PATH, after_init_config=True)
             console.rule('Datasource added')
         else:
             console.rule('Abort to add datasource', style='red')
