@@ -3,33 +3,37 @@ import decimal
 import math
 import time
 from concurrent.futures import ThreadPoolExecutor
+from dataclasses import dataclass
 from datetime import datetime, date, timezone
-from typing import Union, List, Tuple
+from typing import Dict, Optional, Union, List, Tuple
 
+import sentry_sdk
 from dateutil.relativedelta import relativedelta
 from sqlalchemy import MetaData, Table, Column, String, Integer, Numeric, Date, DateTime, Boolean, ARRAY, select, func, \
     distinct, case, text, literal_column, inspect, JSON
 from sqlalchemy.dialects.postgresql import UUID
 from sqlalchemy.engine import Engine, Connection
-from sqlalchemy.pool import SingletonThreadPool
 from sqlalchemy.sql import FromClause, Selectable
 from sqlalchemy.sql.elements import ColumnClause
 from sqlalchemy.sql.expression import CTE, false, true, table as table_clause, column as column_clause
 from sqlalchemy.types import Float
+from sqlalchemy.exc import NoSuchTableError
 
 from .event import ProfilerEventHandler, DefaultProfilerEventHandler
 from ..configuration import Configuration
 from ..datasource import DataSource
+from ..event import capture_exception
 
 HISTOGRAM_NUM_BUCKET = 50
 
 
 class ProfileSubject:
-    def __init__(self, table: str, schema: str = None, database: str = None, name: str = None):
+    def __init__(self, table: str, schema: str = None, database: str = None, name: str = None, ref_id: str = None):
         self.table = table
         self.schema = schema
         self.database = database
         self.name = name if name else table
+        self.ref_id = ref_id
 
 
 def dtof(value: Union[int, float, decimal.Decimal]) -> Union[int, float]:
@@ -89,6 +93,42 @@ async def _run_in_executor(executor, func, *args):
         return func(*args)
 
 
+@dataclass
+class CollectedMetadata:
+    map_name_tables: dict
+    profiled_tables: dict
+    result: dict
+    subjects: List[ProfileSubject]
+
+
+def transform_as_run(profiled_tables) -> Dict:
+    from collections import Counter
+
+    def _t(k):
+        if k is None:
+            return "__no_such_table__"
+        return k.split(".")[-1]
+
+    c = Counter([_t(ref_id) for ref_id in profiled_tables.keys()])
+    conflicts = [k for k, v in c.items() if v > 1]
+
+    if not conflicts:
+        return {_t(k): v for k, v in profiled_tables.items()}
+
+    rebuilt_profiled_tables = {}
+    for ref_id, tresult in profiled_tables.items():
+        table_name = _t(ref_id)
+        if table_name not in conflicts:
+            rebuilt_profiled_tables[table_name] = tresult
+            continue
+
+        rebuilt_profiled_tables[ref_id] = tresult
+        if tresult:
+            rebuilt_profiled_tables[table_name] = tresult
+
+    return rebuilt_profiled_tables
+
+
 class Profiler:
     """
     Profiler profile tables and columns by a sqlalchemy engine.
@@ -103,6 +143,7 @@ class Profiler:
         self.data_source = data_source
         self.event_handler = event_handler
         self.config = config
+        self.collected_metadata: Optional[CollectedMetadata] = None
         if self.data_source.threads > 1:
             self.executor = ThreadPoolExecutor(max_workers=self.data_source.threads)
         else:
@@ -117,24 +158,33 @@ class Profiler:
         self.event_handler.handle_metadata_start()
         self.event_handler.handle_metadata_progress(total, completed)
         for subject in subjects:
-            def _fetch_table_task(subject):
+            def _fetch_table_task(subject: ProfileSubject):
                 engine = self.data_source.get_engine_by_database(subject.database)
                 schema = subject.schema.lower() if subject.schema is not None else None
-                return subject.name, Table(subject.table, MetaData(), autoload_with=engine, schema=schema)
+                table = None
+                try:
+                    table = Table(subject.table, MetaData(), autoload_with=engine, schema=schema)
+                except NoSuchTableError:
+                    # ignore the table metadata fetch error
+                    pass
+                except Exception as e:
+                    capture_exception(e)
+                return subject, table
 
             future = _run_in_executor(self.executor, _fetch_table_task, subject)
             futures.append(future)
 
         for future in asyncio.as_completed(futures):
-            name, table = await future
-            map_name_tables[name] = table
+            subject, table = await future
+            map_name_tables[subject.ref_id] = table
             completed += 1
             self.event_handler.handle_metadata_progress(total, completed)
         self.event_handler.handle_metadata_end()
 
         return map_name_tables
 
-    async def _profile(self, subjects: List[ProfileSubject] = None) -> dict:
+    async def _profile(self, subjects: List[ProfileSubject] = None, *,
+                       metadata_subjects: List[ProfileSubject] = None) -> dict:
         """
         profile all tables or specific table. With different column types, it would profile different metrics.
 
@@ -150,29 +200,27 @@ class Profiler:
         :return: the profile results
         """
 
-        profiled_tables = {}
-        result = {
-            "tables": profiled_tables,
-        }
+        if self.collected_metadata is None:
+            await self._collect_metadata(subjects, metadata_subjects)
 
-        if subjects is None:
-            subjects = []
-            table_names = inspect(self.data_source.get_engine_by_database()).get_table_names()
-            for table_name in table_names:
-                subject = ProfileSubject(table_name)
-                subjects.append(subject)
+        map_name_tables = self.collected_metadata.map_name_tables
+        profiled_tables = self.collected_metadata.profiled_tables
+        result = self.collected_metadata.result
+        subjects = self.collected_metadata.subjects
 
         table_count = len(subjects)
         table_index = 0
 
+        # Profiling
         if len(subjects) > 0:
             self.event_handler.handle_run_start(result)
             self.event_handler.handle_run_progress(result, table_count, table_index)
 
-            map_name_tables = await self._fetch_metadata(subjects)
             for subject in subjects:
                 name = subject.name
-                table = map_name_tables[name]
+                table = map_name_tables.get(subject.ref_id)
+                if table is None:
+                    continue
                 engine = self.data_source.get_engine_by_database(subject.database)
                 table_profiler = TableProfiler(engine, self.executor, subject, table, self.event_handler, self.config)
                 tresult = await table_profiler.profile()
@@ -181,16 +229,104 @@ class Profiler:
                 self.event_handler.handle_run_progress(result, table_count, table_index)
             self.event_handler.handle_run_end(result)
         else:
-            print("No table to profile")
+            print("No models, seeds, sources to profile")
 
         return result
 
-    def profile(self, subjects: List[ProfileSubject] = None) -> dict:
-        if self.executor:
-            with self.executor:
-                return asyncio.run(self._profile(subjects))
-        else:
-            return asyncio.run(self._profile(subjects))
+    async def _collect_metadata(self, subjects: List[ProfileSubject], metadata_subjects: List[ProfileSubject]):
+        profiled_tables = {}
+        if subjects is None:
+            subjects = []
+            table_names = inspect(self.data_source.get_engine_by_database()).get_table_names()
+            for table_name in table_names:
+                subject = ProfileSubject(table_name)
+                subjects.append(subject)
+
+        # Fetch schema data
+        map_name_tables = await self._fetch_metadata(metadata_subjects if metadata_subjects else subjects)
+
+        if metadata_subjects is None:
+            # for compatible with non-dbt cases, we use subjects as the metadata_subjects
+            metadata_subjects = subjects
+
+        for subject in metadata_subjects:
+            engine = self.data_source.get_engine_by_database(subject.database)
+            table = map_name_tables.get(subject.ref_id)
+
+            if table is not None:
+                table_profiler = TableProfiler(engine, self.executor, subject, table, self.event_handler, self.config)
+                tresult = await table_profiler.fetch_schema()
+                profiled_tables[subject.ref_id] = tresult
+            else:
+                profiled_tables[subject.ref_id] = dict(name=subject.name, columns={}, ref_id=subject.ref_id)
+
+        profiled_tables = transform_as_run(profiled_tables)
+        self.collected_metadata = CollectedMetadata(map_name_tables=map_name_tables,
+                                                    profiled_tables=profiled_tables,
+                                                    result=dict(tables=profiled_tables),
+                                                    subjects=subjects)
+        return self.collected_metadata
+
+    def collect_metadata(self, metadata_subjects: List[ProfileSubject], subjects: List[ProfileSubject]):
+        return asyncio.run(self._collect_metadata(subjects, metadata_subjects))
+
+    async def _collect_metadata_from_dbt_manifest(self, dbt_manifest, metadata_subjects, subjects):
+        profiled_tables = {}
+        if subjects is None:
+            subjects = []
+            # table_nodes = filter(
+            #     lambda node: node.startswith('model.') or node.startswith('seed.') or node.startswith('source.'),
+            #     dbt_manifest['nodes'].keys())
+            # print(table_nodes)
+
+        if metadata_subjects is None:
+            metadata_subjects = subjects
+
+        self.event_handler.handle_manifest_start()
+        total = len(metadata_subjects)
+        completed = 0
+        for subject in metadata_subjects:
+            table_name = subject.name
+            ref_id = subject.ref_id
+            dbt_node = dbt_manifest.get('nodes', {}).get(ref_id, {})
+            dbt_source = dbt_manifest.get('sources', {}).get(ref_id, {})
+            dbt_columns = {}
+            columns = {}
+            if dbt_node:
+                # Read columns from dbt manifest `columns` field
+                dbt_columns = dbt_node.get('columns', {})
+            elif dbt_source:
+                # Read columns from dbt manifest `sources` field
+                dbt_columns = dbt_source.get('columns', {})
+
+            # Fill the columns with name and description
+            for key, val in dbt_columns.items():
+                name = val.get('name')
+                description = val.get('description')
+                columns[key] = dict(
+                    name=name,
+                    type='other',
+                    schema_type='',
+                    description=description)
+            profiled_tables[ref_id] = dict(name=table_name, columns=columns, ref_id=ref_id)
+            completed += 1
+            self.event_handler.handle_manifest_progress(total, completed)
+
+        self.event_handler.handle_manifest_end()
+        return dict(tables=profiled_tables)
+
+    def collect_metadata_from_dbt_manifest(self, dbt_manifest, metadata_subjects: List[ProfileSubject],
+                                           subjects: List[ProfileSubject]):
+        return asyncio.run(self._collect_metadata_from_dbt_manifest(dbt_manifest, metadata_subjects, subjects))
+
+    def profile(self, subjects: List[ProfileSubject] = None, *, metadata_subjects: List[ProfileSubject] = None) -> dict:
+        def job():
+            return asyncio.run(self._profile(subjects, metadata_subjects=metadata_subjects))
+
+        if not self.executor:
+            return job()
+        with self.executor:
+            return job()
 
 
 class TableProfiler:
@@ -411,14 +547,30 @@ class TableProfiler:
         await _run_in_executor(self.executor, self._profile_table_duplicate_rows, result)
 
     async def _profile_column(self, result, table_name, table: Table, column: Column) -> dict:
-        profiler_config = self.config.profiler_config if self.config else {}
         column_name = column.name
+        column_result, profiler = await self._create_column_metadata_and_profiler(table, column)
+
+        self.event_handler.handle_column_start(table_name, column_name)
+
+        profile_start = time.perf_counter()
+        profile_result = await _run_in_executor(self.executor, profiler.profile)
+        profile_end = time.perf_counter()
+        duration = profile_end - profile_start
+
+        column_result.update(profile_result)
+        column_result["profile_duration"] = f"{duration:.2f}"
+        column_result["elapsed_milli"] = int(duration * 1000)
+
+        self.event_handler.handle_column_end(table_name, column_name, column_result)
+        result['columns'][column_name] = column_result
+
+    async def _create_column_metadata_and_profiler(self, table, column):
+        profiler_config = self.config.profiler_config if self.config else {}
         column_type = column.type
         schema_type = str(column.type)
         if isinstance(column_type, ARRAY) and column_type.item_type is not None:
             column_type = column_type.item_type
             schema_type = f"ARRAY<{column_type}>"
-
         if isinstance(column_type, String):
             # VARCHAR
             # CHAR
@@ -455,32 +607,16 @@ class TableProfiler:
         else:
             generic_type = "other"
             profiler = BaseColumnProfiler(self.engine, profiler_config, table, column)
-
         column_result = {
             "name": column.name,
             "type": generic_type,
             "schema_type": schema_type,
         }
-
-        self.event_handler.handle_column_start(table_name, column_name)
-
-        profile_start = time.perf_counter()
-        profile_result = await _run_in_executor(self.executor, profiler.profile)
-        profile_end = time.perf_counter()
-        duration = profile_end - profile_start
-
-        column_result.update(profile_result)
-        column_result["profile_duration"] = f"{duration:.2f}"
-        column_result["elapsed_milli"] = int(duration * 1000)
-
-        self.event_handler.handle_column_end(table_name, column_name, column_result)
-
-        result['columns'][column_name] = column_result
+        return column_result, profiler
 
     async def profile(self) -> dict:
         subject = self.subject
         name = subject.name
-
         self.event_handler.handle_table_start(name)
         candidate_columns = list(self._get_candidate_columns())
         col_index = 0
@@ -494,8 +630,10 @@ class TableProfiler:
             "col_count": col_count,
             "duplicate_rows": None,
             "duplicate_rows_p": None,
-            "columns": columns
+            "columns": columns,
         }
+        if subject.ref_id:
+            result['ref_id'] = subject.ref_id
         futures = []
         profile_start = time.perf_counter()
 
@@ -532,6 +670,34 @@ class TableProfiler:
         self.event_handler.handle_table_end(name, result)
         return result
 
+    async def fetch_schema(self) -> dict:
+        subject = self.subject
+        name = subject.name
+        candidate_columns = list(self._get_candidate_columns())
+        col_count = len(candidate_columns)
+        columns = {}
+        result = {
+            "name": name,
+            "col_count": col_count,
+            "columns": columns,
+        }
+
+        if subject.ref_id:
+            result['ref_id'] = subject.ref_id
+
+        futures = []
+
+        # Profile columns
+        for selectable, column in candidate_columns:
+            columns[column.name] = None
+            future = asyncio.create_task(self._create_column_metadata_and_profiler(selectable, column))
+            futures.append(future)
+
+        for future in asyncio.as_completed(futures):
+            column_result, _ = await future
+            columns[column_result['name']] = column_result
+        return result
+
 
 class BaseColumnProfiler:
     """
@@ -861,13 +1027,13 @@ class NumericColumnProfiler(BaseColumnProfiler):
 
             # histogram
             histogram = None
-            if _valids > 0:
+            if _valids > 0 and math.isfinite(_min) and math.isfinite(_max):
                 histogram = profile_histogram(conn, cte, cte.c.c, _min, _max, self.is_integer)
             result['histogram'] = histogram
 
             # quantile
             quantile = {}
-            if _valids > 0:
+            if _valids > 0 and math.isfinite(_min) and math.isfinite(_max):
                 quantile = self._profile_quantile(conn, cte, cte.c.c, _valids)
             result.update({
                 'p5': quantile.get('p5'),
@@ -1269,7 +1435,10 @@ class DatetimeColumnProfiler(BaseColumnProfiler):
             _type = "monthly"
             interval = relativedelta(months=+1)
             dmin = date(min.year, min.month, 1)
-            dmax = date(max.year, max.month, 1) + interval
+            if max.year < 3000:
+                dmax = date(max.year, max.month, 1) + interval
+            else:
+                dmax = date(3000, 1, 1)
             period = relativedelta(dmax, dmin)
             num_buckets = (period.years * 12 + period.months)
             cte = select(date_trunc("MONTH", column).label("d")).select_from(table).cte()
@@ -1277,7 +1446,10 @@ class DatetimeColumnProfiler(BaseColumnProfiler):
             _type = "daily"
             interval = relativedelta(days=+1)
             dmin = date(min.year, min.month, min.day)
-            dmax = date(max.year, max.month, max.day) + interval
+            if max.year < 3000:
+                dmax = date(max.year, max.month, max.day) + interval
+            else:
+                dmax = date(3000, 1, 1)
             num_buckets = (dmax - dmin).days
             cte = select(date_trunc("DAY", column).label("d")).select_from(table).cte()
 
